from gettext import gettext as _

import gedit

import gtk
import os.path
import gobject
import sys
import StringIO
import thread

import bzrlib.errors
from bzrlib import workingtree
from bzrlib import branch
from bzrlib import bzrdir
from bzrlib import merge
from bzrlib import globbing
from bzrlib import transport
from bzrlib import log
from bzrlib import check
from bzrlib import osutils

try:
    from bzrlib.plugins import gtk as bzrgtk

except:
    bzrgtk = False


from bzrlib.progress import DummyProgress
from bzrlib.plugin import load_plugins
load_plugins() #right now only used to make "branch lp:bla" work


import dialogs

ui_str = """<ui>
  <menubar name="MenuBar">
    <menu name="ToolsMenu" action="Tools">
      <placeholder name="BazaarMenu">

        <menu name="BazaarMenu" action="BazaarMenu">
            <menuitem name="BazaarInit" action="BazaarInit"/>
            <menuitem name="BazaarBranchFrom" action="BazaarBranchFrom"/>
            <separator/>
            <menuitem name="BazaarCommit" action="BazaarCommit"/>
            <menu name="BazaarAdd" action="BazaarAdd">
                <menuitem name="BazaarAddAll" action="BazaarAddAll"/>
                <menuitem name="BazaarAddCurrent" action="BazaarAddCurrent"/>
            </menu>
            <menuitem name="BazaarPush" action="BazaarPush"/>
            <menuitem name="BazaarPull" action="BazaarPull"/>
            <menuitem name="BazaarMerge" action="BazaarMerge"/>
            <menuitem name="BazaarUpdate" action="BazaarUpdate"/>
            <menuitem name="BazaarStatus" action="BazaarStatus"/>

            <menuitem name="BazaarMVFile" action="BazaarMVFile"/>
            
            <menu name="BazaarDiff" action="BazaarDiff">
                <menuitem name="BazaarDiffCurrent" action="BazaarDiffCurrent"/>
                <menuitem name="BazaarDiffBranch" action="BazaarDiffBranch"/>
            </menu>
            <menu name="BazaarLog" action="BazaarLog">
                <menuitem name="BazaarLogFile" action="BazaarLogFile"/>
                <menuitem name="BazaarLogDir" action="BazaarLogDir"/>
                <menuitem name="BazaarLogBranch" action="BazaarLogBranch"/>
            </menu>            
            <menuitem name="BazaarRevert" action="BazaarRevert"/>
            <menuitem name="BazaarIgnoreCurrent" action="BazaarIgnoreCurrent"/>
            <menuitem name="BazaarViz" action="BazaarViz"/>
            
            <menuitem name="BazaarCheck" action="BazaarCheck"/>


            <menuitem name="BazaarConflicts" action="BazaarConflicts"/>
            <menuitem name="BazaarPatch" action="BazaarPatch"/>
            <menuitem name="BazaarTags" action="BazaarTags"/>
            <menuitem name="BazaarConfig" action="BazaarConfig"/>
        </menu>
      </placeholder>
    </menu>
  </menubar>
  <toolbar name="ToolBar">
    <separator/>
    <toolitem name="BazaarToolButton" action="BazaarToolButton"/>
  </toolbar>
</ui>
"""

class MenuToolAction(gtk.Action):
    __gtype_name__ = "MenuToolAction"

gobject.type_register(MenuToolAction)
MenuToolAction.set_tool_item_type(gtk.MenuToolButton)

class BazaarWindowHelper:
    _statusframe = None
    _statuslbl = None
    _status_img = None

    _menu_paths_bzr = ["/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarCommit",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarAdd/BazaarAddAll",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarAdd/BazaarAddCurrent",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarPush",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarPull",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarMerge",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarUpdate",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarStatus",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarMVFile",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarDiff/BazaarDiffCurrent",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarDiff/BazaarDiffBranch",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarLog/BazaarLogFile",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarLog/BazaarLogDir",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarLog/BazaarLogBranch",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarRevert",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarIgnoreCurrent",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarViz",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarCheck",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarConflicts",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarPatch",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarTags",
                    
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarConfig"                 
                    ]
    _menu_paths_nobzr = ["/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarInit",
                    "/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarBranchFrom"]


    def __init__(self, plugin, window):

        self._mypath = os.path.dirname(__file__)
        self._window = window
        self._plugin = plugin
        self._insert_menu()
        self.update_ui()
        
    def _get_active_doc_uri_exists(self):
        doc = self._window.get_active_document()
        if doc != None:
            docuri = doc.get_uri_for_display()
            if os.path.exists(docuri):
                return docuri
            else:
                return None
        else:
            return None
        
    # UI Callbacks
    def on_bazaar_init(self, menu):

        docuri = self._get_active_doc_uri_exists()
        if docuri is not None:
            if not self._get_is_active_branch(docuri):
                docpath = os.path.dirname(docuri)
                inputdlg = dialogs.BzrInputDialog(docpath,"Select path to INIT","Choose which is the path where you want to create this bzr")
                code, newdocpath = inputdlg.run()

                if code == gtk.RESPONSE_OK and newdocpath != '' and os.path.exists(newdocpath) and os.path.isdir(newdocpath) and not self._get_is_active_branch(docuri):
                    try:
                        wt = bzrlib.bzrdir.BzrDir.create_standalone_workingtree(newdocpath)
                    except:
                        dialogs._bzrErrorDialog("Bazaar error executing Init",sys.exc_value)
                    else:
                        self.update_ui()
                    self._statusbar_draw("Directory %s innited" % newdocpath)

                elif newdocpath == '':
                    dialogs._bzrErrorDialog("The path you entered is incorrect")
                elif not os.path.exists(newdocpath) or not os.path.isdir(newdocpath):
                    dialogs._bzrErrorDialog("Can't Init. The path you choosed doesn't exist")
                elif self._get_is_active_branch(docuri):
                    dialogs._bzrErrorDialog("Can't Init. The path you choosed is already a Bazaar branch")
                inputdlg.destroy()
            else:
                dialogs._bzrErrorDialog("Can't Init. This file is already in a Bazaar branch")
        else:
            dialogs._bzrErrorDialog("Can't Init. You have to save your file before doing init")


    def on_bazaar_branchfrom(self, menu):

        localpath = self._get_active_doc_dir()
        if localpath is None:
            localpath = ''

        bigtext = "Enter branch and local path info"
        text1 = "Enter branch location"
        text2 = "Choose path where you want to creat your local branch"
        inputdlg = dialogs.BzrDobleInputDialog("",localpath,bigtext, text1, text2, "")
        response,remotebranch, localpath = inputdlg.run()

        inputdlg.disable_widgets()
        if response == gtk.RESPONSE_OK:
            if localpath != "":
                if remotebranch != '':
                    try:
                        inputdlg.set_status("Opening remote branch...")
                        onlinebranch = branch.Branch.open(remotebranch)
                    except :
                        dialogs._bzrErrorDialog("Bazaar error branching",sys.exc_value)
                    else:
                        try:
                            inputdlg.set_status("Saving local copy...")
                            localbranch = onlinebranch.bzrdir.sprout(localpath)

                        except:
                            dialogs._bzrErrorDialog("Bazaar error saving local copy",sys.exc_value)
                        else:
                            self._statusbar_draw("Branch created int %s" % localpath)
                else:
                    dialogs._bzrErrorDialog("Branch location can't be empty")
            else:
                dialogs._bzrErrorDialog("Local folder can't be empty")
        inputdlg.destroy()


    def on_bazaar_commit(self, menu):
        docuri = self._get_active_doc_uri_exists()
        if docuri is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt,relpath = workingtree.WorkingTree.open_containing(docuri)
            except:
                dialogs._bzrErrorDialog("Bazaar error doing commit",sys.exc_value)
            else:
                commitdlg = dialogs.BzrComitDialog(wt.basedir)
                response, comment = commitdlg.run()
                commitdlg.disable_widgets()
                if response == gtk.RESPONSE_OK:
                    try:
                        commitdlg.set_status("Committing...")
                        wt.commit(comment)
                    except:
                        dialogs._bzrErrorDialog("Bazaar error doing commit",sys.exc_value)
                    else:
                        self._statusbar_draw("Commit on  %s ok" % wt.basedir) # FIXME: should say also revision num?
                commitdlg.destroy()


    def on_bazaar_add(self, menu):
        
        localpath = self._get_active_doc_dir()
        if localpath is None:
            localpath = ''

        if localpath == '':
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt,relpath = workingtree.WorkingTree.open_containing(localpath)
            except:
                dialogs._bzrErrorDialog("Bazaar error opening %s to add." % localpath,sys.exc_value)
            else:
                try:
                    ret = wt.smart_add([wt.basedir])
                except:
                    dialogs._bzrErrorDialog("Bazaar error adding files in %s" % localpath,sys.exc_value)
                else:
                    self._statusbar_draw("Files added in %s" % localpath)

    def on_bazaar_add_current(self,menu):
        docuri = self._get_active_doc_uri_exists()
        if docuri is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt,relpath = workingtree.WorkingTree.open_containing(docuri)
            except:
                dialogs._bzrErrorDialog("Bazaar error opening %s to add." % docuri,sys.exc_value)
            else:
                try:
                    ret = wt.smart_add([docuri])
                except:
                    dialogs._bzrErrorDialog("Bazaar error adding %s" % docuri,sys.exc_value)
                else:
                    self._statusbar_draw("%s added" % docuri)
    
    def on_bazaar_push(self, menu):      
        localpath = self._get_active_doc_dir()
        if localpath is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                br,relpath = branch.Branch.open_containing(localpath)
            except:
                dialogs._bzrErrorDialog("Bazaar error pushing %s" % localpath,sys.exc_value)
            else:
                pushloc = br.get_push_location()
                if pushloc == None:
                    pushloc = ''
                #bzr+ssh://javierder@bazaar.launchpad.net/%7Ejavierder/bzr-gedit/testing
                inputdlg = dialogs.BzrInputDialog(pushloc,"Push your changes", "Enter the location where you want to push your branch","Push")
                response, pushlocation = inputdlg.run()
                inputdlg.disable_widgets()
                # FIXME: previous dialog yould also allow to save new location...
                if response == gtk.RESPONSE_OK and pushlocation != '':
                    
                    # Fix to bug #213515
                    # first, get a transport
                    to_transport = transport.get_transport(pushlocation)
                    # then, test if there's a dir in remote location
                    
                    inputdlg.set_status("Opening remote location")
                    remote_branch_ok = False
                    try:
                        branch_dest = branch.Branch.open(pushlocation)
                    except bzrlib.errors.NotBranchError:
                        inputdlg.set_status("Creating dir in remote location")
                        # it doesn't exist. Create.
                        try:
                            br.bzrdir.clone_on_transport(to_transport)
                        except:
                            dialogs._bzrErrorDialog("Bazaar error creating remote branch(%s)" % pushlocation,sys.exc_value)
                        else:
                            inputdlg.set_status("Opening remote location")
                            try:
                                branch_dest = branch.Branch.open(pushlocation)
                            except:
                                dialogs._bzrErrorDialog("Bazaar error opening destination branch(%s)" % pushlocation,sys.exc_value)                            
                            else:
                                remote_branch_ok = True
                    except:
                        dialogs._bzrErrorDialog("Bazaar error opening destination branch(%s)" % pushlocation,sys.exc_value)
                    else:
                        remote_branch_ok = True
                    
                   
                    if remote_branch_ok:
                        try:
                            inputdlg.set_status("Pushing...")
                            res = br.push(branch_dest)
                        except:
                            dialogs._bzrErrorDialog("Bazaar error pushing" ,sys.exc_value)
                        else:
                            self._statusbar_draw("Branch pushed to %s" % pushlocation)
                            
                elif pushlocation == '' and response == gtk.RESPONSE_OK:
                    dialogs._bzrErrorDialog("Error. You have to enter a push location.")
                inputdlg.destroy()

    def on_bazaar_pull(self, menu):

        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                br,relpath = branch.Branch.open_containing(localpath)
            except:
                dialogs._bzrErrorDialog("Bazaar error opening %s" % localpath ,sys.exc_value)
            else:
                pullloc = br.get_parent()
                if pullloc == None:
                    pullloc = ''
                inputdlg = dialogs.BzrInputDialog(pullloc,"Pull from parent", "Enter the location from where you want to pull code","Pull")
                response, pulllocation = inputdlg.run()
                inputdlg.disable_widgets()
                # FIXME: this dialog yould allow to force "update" and ?to save new location?
                if response == gtk.RESPONSE_OK and pulllocation != '':
                    try:
                        inputdlg.set_status("Opening remote branch...")
                        branch_remote = branch.Branch.open(pulllocation)
                    except:
                        dialogs._bzrErrorDialog("Bazaar error opening %s" % pulllocation ,sys.exc_value)
                    else:
                        try:
                            inputdlg.set_status("Pulling...")
                            res = br.pull(branch_remote)

                        except:
                            dialogs._bzrErrorDialog("Bazaar error pulling from %s" % pulllocation ,sys.exc_value)
                        else:
                            self._statusbar_draw("Branch pulled from %s " % pulllocation)
                elif response == gtk.RESPONSE_OK and pulllocation == '':
                    dialogs._bzrErrorDialog("Error. You have to enter a pull location.")
                inputdlg.destroy()

    def on_bazaar_merge(self, menu):
        
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt,relpath = workingtree.WorkingTree.open_containing(localpath)
            except:
                dialogs._bzrErrorDialog("Bazaar error opening %s" % localpath ,sys.exc_value)
            else:
                mergeloc = wt.branch.get_parent()
                if mergeloc == None:
                    mergeloc = ''
                inputdlg = dialogs.BzrInputDialog(mergeloc,"Merge from", "Enter the location from where you want to merge code","Merge")
                response, mergelocation = inputdlg.run()
                inputdlg.disable_widgets()
                # FIXME: this dialog yould allow to force "update" and ?to save new location?
                # and select revision, allow merge bundles...
                if response == gtk.RESPONSE_OK and mergelocation != '':
                    try:
                        inputdlg.set_status("Opening remote branch...")
                        
                        branch_remote = branch.Branch.open(mergelocation)
                    except:
                        dialogs._bzrErrorDialog("Bazaar error opening %s" % mergelocation ,sys.exc_value)
                    else:
                        try:
                            inputdlg.set_status("Merging...")
                            merger = merge.Merger.from_revision_ids(DummyProgress(),wt,branch_remote.last_revision(),wt.branch.last_revision())
                            merger.merge_type = merge.Merge3Merger
                            if merger.other_rev_id == merger.base_rev_id:
                                dialogs._bzrErrorDialog("Can't merge. Nothing to do")
                            else:
                                ret = merger.do_merge()
                                if ret != 0:
                                    dialogs._bzrErrorDialog("This merge caused conflicts")
                                    # FIXME: so, now WHAT?
                                    
                        except:
                            dialogs._bzrErrorDialog("Bazaar error merging from %s" % mergelocation ,sys.exc_value)
                        else:
                            self._statusbar_draw("Branch merged from %s " % mergelocation)
                            # FIXME: should force update?
                elif response == gtk.RESPONSE_OK and mergelocation == '':
                    dialogs._bzrErrorDialog("Error. You have to enter a merge location.")
                inputdlg.destroy()

    
    def on_bazaar_update(self, menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt,relpath = workingtree.WorkingTree.open_containing(localpath)
            except:
                dialogs._bzrErrorDialog("Bazaar error updating." ,sys.exc_value)
            else:
                wt.update()
                self._statusbar_draw("Updated %s" % localpath)

    def on_bazaar_status(self,menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt ,relpath = workingtree.WorkingTree.open_containing(localpath)
                old_tree = wt.branch.repository.revision_tree(wt.branch.last_revision())
                delta = wt.changes_from(old_tree)
                if delta.has_changed():
                    dialogs._bzrInfoDialog("Status",delta.get_changes_as_text(),[250,300])
                else:
                    dialogs._bzrErrorDialog("This branch has no changes")
            except:
                dialogs._bzrErrorDialog("Bazaar error showing status." ,sys.exc_value)            
    
    def on_bazaar_move(self, menu):
        docuri = self._get_active_doc_uri_exists()
        if docuri is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt,relpath = workingtree.WorkingTree.open_containing(docuri)
                
    
                inputdlg = dialogs.BzrInputDialog(wt.basedir,"Move file ./%s to:" % relpath, "Select where you want to move your file","Move file")
                retcode, newpath = inputdlg.run()
                
                names_list = [docuri,newpath]
                rel_names = [wt.relpath(docuri),wt.relpath(newpath)]
                after = False
    
                into_existing = osutils.isdir(names_list[-1])
                tree = wt
    
                wt.lock_write()
                
                if into_existing and len(names_list) == 2:
                    inv = tree.inventory
                    from_id = tree.path2id(rel_names[0])
                    if (not osutils.lexists(names_list[0]) and
                        from_id and inv.get_file_kind(from_id) == "directory"):
                        into_existing = False
                # move/rename
                if into_existing:
                    # move into existing directory
                    for pair in tree.move(rel_names[:-1], rel_names[-1], after=after):
                        print("%s => %s\n" % pair)
                else:
                    if len(names_list) != 2:
                        raise errors.BzrCommandError('to mv multiple files the'
                                                     ' destination must be a versioned'
                                                     ' directory')
                    tree.rename_one(rel_names[0], rel_names[1], after=after)
                    print ("%s => %s\n" % (rel_names[0], rel_names[1]))
                wt.unlock()
                ###
            except:
                wt.unlock()
                dialogs._bzrErrorDialog("Bazaar error opening %s to add." % docuri,sys.exc_value)
            inputdlg.destroy()

    def on_bazaar_diff(self,menu,type):
        docuri = self._get_active_doc_uri_exists()
        if docuri is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                new_tree ,relpath = workingtree.WorkingTree.open_containing(docuri)
                old_revid = new_tree.branch.revision_history()[-2]
                old_tree = new_tree.branch.repository.revision_tree(old_revid)
                
                from bzrlib.diff import _get_trees_to_diff, show_diff_trees
                
                to_file = StringIO.StringIO()
                
                if type == 'branch':
                    relpath = ''
                
                show_diff_trees(old_tree, new_tree, to_file,
                                specific_files=[relpath,])
                
                dialogs._bzrDiffDialog("File diff: %s" % relpath,to_file.getvalue())
            except:
                dialogs._bzrErrorDialog("Bazaar error showing diff." ,sys.exc_value)            
            
    def on_bazaar_log(self,action,log_type):

        docuri = self._get_active_doc_uri_exists()
        if docuri is None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                if log_type != 'dir':
                    br ,relpath = branch.Branch.open_containing(docuri)
                else:
                    br ,relpath = branch.Branch.open_containing(self._get_active_doc_dir())
                to_file = StringIO.StringIO()
                lf = log.LongLogFormatter(to_file=to_file)
                if log_type == 'branch':
                    log.show_log(br, lf)
                    # FIXME: this should have a "find" function...
                else:
                    tree = br.bzrdir.open_workingtree()
                    file_id = tree.path2id(relpath)
                    if file_id == 'TREE_ROOT':
                        log.show_log(br, lf) # if it's tree root let's show branch
                    else:
                        log.show_log(br, lf,file_id)
                        
                dialogs._bzrInfoDialog("Branch Log",to_file.getvalue(),[400,250])
                
            except:
                dialogs._bzrErrorDialog("Bazaar error showing log." ,sys.exc_value)

        
        
    def on_bazaar_revert(self,menu):
        localpath = self._get_active_doc_dir()
        if localpath == None:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
        else:
            try:
                wt,relpath = workingtree.WorkingTree.open_containing(localpath)
            except:
                dialogs._bzrErrorDialog("Bazaar error doing revert." ,sys.exc_value)
            else:
                try:
                    ret = wt.revert()
                except:
                    dialogs._bzrErrorDialog("Bazaar error doing revert." ,sys.exc_value)
                else:
                    self._statusbar_draw("Reverted all files in %s" % localpath)

    def on_bazaar_ignorecurrent(self,menu):
        docuri = self._get_active_doc_uri_exists()
        if docuri:
            response = dialogs._bzrQuestionDialog("Ignore","Make bazaar ignore <b>%s</b>?" % docuri,"Ignore file")
            if response == gtk.RESPONSE_YES:
                wt, name_pattern = workingtree.WorkingTree.open_containing(docuri)
                if wt.is_ignored(docuri) is None:
                    from bzrlib.atomicfile import AtomicFile
                    
                    ifn = wt.abspath('.bzrignore')
                    if os.path.exists(ifn):
                        f = open(ifn, 'rt')
                        try:
                            igns = f.read().decode('utf-8')
                        finally:
                            f.close()
                    else:
                        igns = ''
                    if igns and igns[-1] != '\n':
                        igns += '\n'
                    
                    igns += name_pattern + '\n'
                    
                    f = AtomicFile(ifn, 'wb')
                    try:
                        f.write(igns.encode('utf-8'))
                        f.commit()
                    finally:
                        f.close()
            
                    if not wt.path2id('.bzrignore'):
                        wt.add(['.bzrignore'])
                        
                    ignored = globbing.Globster([name_pattern])
                    matches = []
                    wt.lock_read()
                    
                    for entry in wt.list_files():
                        id = entry[3]
                        if id is not None:
                            filename = entry[0]
                            if ignored.match(filename):
                                matches.append(filename.encode('utf-8'))
                    wt.unlock()
                    if len(matches) > 0:
                        dialogs._bzrErrorDialog("The following files are version controlled and" \
                              " match your ignore pattern:\n%s" % ("\n".join(matches),),msgtype='Warning')
                    self._statusbar_draw("File added to ignore list")
                else:
                    dialogs._bzrErrorDialog("This file is already in the ignore list")
        else:
            dialogs._bzrErrorDialog("This action must be done from a saved document.")
            
    def on_bazaar_viz(self, menu):
        if self._check_bzrgtk("visualize"):
            localpath = self._get_active_doc_dir()
            if localpath == None:
                dialogs._bzrErrorDialog("This action must be done from a saved document.")
            else:
                try:
                    br, relpath = branch.Branch.open_containing(localpath)
                    rv = br.last_revision() # FIXME: we should allow choosing a revision
                    vizwin = bzrgtk.start_viz_window(br,rv,None)
                    vizwin.show()
                except:
                    dialogs._bzrErrorDialog("Bazaar error loading visualize plugin.", sys.exc_value)

    def on_bazaar_check(self,menu):
            localpath = self._get_active_doc_dir()
            if localpath == None:
                dialogs._bzrErrorDialog("This action must be done from a saved document.")
            else:
                try:
                    br, relpath = branch.Branch.open_containing(localpath)
                    checkobj = check.Check(br.repository)
                    checkobj.check()
                    
                    checkreport = "Checked %s" % localpath
                    checkreport = checkreport + "\n" + ('%6d revisions' % checkobj.checked_rev_cnt)
                    checkreport = checkreport + "\n" + ('%6d file-ids' % len(checkobj.checked_weaves))
                    checkreport = checkreport + "\n" + ('%6d unique file texts' % checkobj.checked_text_cnt)
                    checkreport = checkreport + "\n" + ('%6d repeated file texts' % checkobj.repeated_text_cnt)
                    checkreport = checkreport + "\n" + ('%6d unreferenced text versions' %
                         len(checkobj.unreferenced_versions))
                    if checkobj.missing_inventory_sha_cnt:
                        checkreport = checkreport + "\n" + ('%6d revisions are missing inventory_sha1' %
                             checkobj.missing_inventory_sha_cnt)
                    if checkobj.missing_revision_cnt:
                        checkreport = checkreport + "\n" + ('%6d revisions are mentioned but not present' %
                             checkobj.missing_revision_cnt)
                    if len(checkobj.ghosts):
                        checkreport = checkreport + "\n" + ('%6d ghost revisions' % len(checkobj.ghosts))
                        """if verbose:
                            for ghost in checkobj.ghosts:
                                checkreport = checkreport + "\n" + ('      %s', ghost)"""
                    if len(checkobj.missing_parent_links):
                        checkreport = checkreport + "\n" + ('%6d revisions missing parents in ancestry' %
                             len(checkobj.missing_parent_links))
                        """if verbose:
                            for link, linkers in checkobj.missing_parent_links.items():
                                checkreport = checkreport + "\n" + ('      %s should be in the ancestry for:', link)
                                for linker in linkers:
                                    checkreport = checkreport + "\n" + ('       * %s', linker)
                        if verbose:
                            for file_id, revision_id in checkobj.unreferenced_versions:
                                log_error('unreferenced version: {%s} in %s', revision_id,
                                    file_id)"""
                    if len(checkobj.inconsistent_parents):
                        checkreport = checkreport + "\n" + ('%6d inconsistent parents' % len(checkobj.inconsistent_parents))
                        """if verbose:
                            for info in checkobj.inconsistent_parents:
                                revision_id, file_id, found_parents, correct_parents = info
                                checkreport = checkreport + "\n" + ('      * %s version %s has parents %r '
                                     'but should have %r'
                                     % (file_id, revision_id, found_parents,
                                         correct_parents))"""
                    if checkobj.revs_with_bad_parents_in_index:
                        checkreport = checkreport + "\n" + ('%6d revisions have incorrect parents in the revision index' %
                             len(checkobj.revs_with_bad_parents_in_index))
                        """if verbose:
                            for item in checkobj.revs_with_bad_parents_in_index:
                                revision_id, index_parents, actual_parents = item
                                checkreport = checkreport + "\n" + (
                                    '       %s has wrong parents in index: '
                                    '%r should be %r',
                                    revision_id, index_parents, actual_parents)"""
                    dialogs._bzrInfoDialog("Repository check",checkreport,[200,300])
                except:
                    dialogs._bzrErrorDialog("Bazaar error doing check.", sys.exc_value)
                    
         
           
    def on_bazaar_conflicts(self,menu):
        if self._check_bzrgtk("conflicts"):
            localpath = self._get_active_doc_dir()
            
            if localpath == None:
                dialogs._bzrErrorDialog("This action must be done from a saved document.")
            else:
                try:
                    (wt, path) = workingtree.WorkingTree.open_containing(localpath)
                    from bzrlib.plugins.gtk.conflicts import ConflictsDialog
                    dialog = ConflictsDialog(wt)
                    dialog.run()    
                except:
                    dialogs._bzrErrorDialog("Bazaar error viewing conflicts.", sys.exc_value)

    def on_bazaar_patch(self,menu):
        if self._check_bzrgtk("patch"):
            localpath = self._get_active_doc_dir()
            if localpath == None:
                dialogs._bzrErrorDialog("This action must be done from a saved document.")
            else:
                try:
                    inputdlg = dialogs.BzrInputFileDialog("","Patch branch","Select patch file","Patch")
                    ret, patchfile = inputdlg.run()
                    if ret == gtk.RESPONSE_OK:
                        if patchfile == '':
                            dialogs._bzrErrorDialog("Bazaar error doing patch.", "You must select a file")
                        else:
                            from bzrlib.plugins.gtk.diff import (DiffWindow,
                                                                 MergeDirectiveWindow)
                            from bzrlib import merge_directive
                            lines = open(patchfile, 'rb').readlines()
                            lines = [l.replace('\r\n', '\n') for l in lines]
                            try:
                                directive = merge_directive.MergeDirective.from_lines(lines)
                            except bzrlib.errors.NotAMergeDirective:
                                window = DiffWindow()
                                window.set_diff_text(patchfile, lines)
                            else:
                                window = MergeDirectiveWindow(directive, patchfile)
                                window.set_diff_text(patchfile, directive.patch.splitlines(True))
                            window.show()
                except:
                    dialogs._bzrErrorDialog("Bazaar error doing patch.", sys.exc_value)
                inputdlg.destroy()
                
    def on_bazaar_tags(self,menu):
        if self._check_bzrgtk("config"):
            localpath = self._get_active_doc_dir()
            if localpath == None:
                dialogs._bzrErrorDialog("This action must be done from a saved document.")
            else:
                try:
                    br, path = branch.Branch.open_containing(localpath)
                    from bzrlib.plugins.gtk.tags import TagsWindow
                    window = TagsWindow(br)
                    window.show()
                except:
                    dialogs._bzrErrorDialog("Bazaar error opening tags window.", sys.exc_value)                    
                    
                    
                    
    def on_bazaar_config(self,menu):
        if self._check_bzrgtk("config"):
            localpath = self._get_active_doc_dir()
            if localpath == None:
                dialogs._bzrErrorDialog("This action must be done from a saved document.")
            else:
                try:
                    from bzrlib.plugins.gtk.preferences import PreferencesWindow
                    dialog = PreferencesWindow()
                    dialog.run()
                except:
                    dialogs._bzrErrorDialog("Bazaar error opening config.", sys.exc_value)                    
                    
    # Gedit callbacks
    def deactivate(self):
        self._remove_menu()
        self._remove_statusbar()

        self._window = None
        self._plugin = None
        self._action_group = None

    def update_ui(self):
        # Called whenever the window has been updated (active tab
        # changed, etc.)
        doc = self._window.get_active_document()

        if doc != None:
            docuri = doc.get_uri_for_display() # FIXME: is this the right one? or should i clean up get_uri?
            if docuri != None and os.path.exists(docuri):
                self._set_menu_sensitive(self._get_is_active_branch(docuri))
            else:
                self._set_menu_sensitive(False)
        else:
            self._set_menu_sensitive(False)

        self._statusbar_draw("")


    # Private ui functions
    def _insert_menu(self):
        # Get the GtkUIManager
        manager = self._window.get_ui_manager()

        # Create a new action group
        self._action_group = gtk.ActionGroup("BazaarActions")

        self._action_group.add_actions([("BazaarMenu", None, _("Bazaar"),
                                         None, _("Bazaar"),
                                         None)])

        #not bazaar dir
        self._action_group.add_actions([("BazaarInit", None, _("Init"),
                                         None, _("Init"),
                                         self.on_bazaar_init)])
        self._action_group.add_actions([("BazaarBranchFrom", None, _("_Branch from..."),
                                         None, _("Branch from..."),
                                         self.on_bazaar_branchfrom)])
        #bazaar dir        
        self._action_group.add_actions([("BazaarCommit", None, _("_Commit"),
                                         None, _("Commit"),
                                         self.on_bazaar_commit)])
        
        self._action_group.add_actions([("BazaarAdd", None, _("_Add files"),
                                         None, _("Add files"),
                                         None)])
        self._action_group.add_actions([("BazaarAddAll", None, _("Add _all..."),
                                         None, _("Add all..."),
                                         self.on_bazaar_add)])
        self._action_group.add_actions([("BazaarAddCurrent", None, _("Add _current file"),
                                         None, _("Add current file"),
                                         self.on_bazaar_add_current)])
        
        self._action_group.add_actions([("BazaarPush", None, _("_Push to..."),
                                         None, _("Push to..."),
                                         self.on_bazaar_push)])
        
       

        #action = action.connect("activate",self.on_bazaar_log,'dir')
        
        
        self._action_group.add_actions([("BazaarPull", None, _("P_ull from..."),
                                         None, _("Pull from..."),
                                         self.on_bazaar_pull)])
        

        self._action_group.add_actions([("BazaarMerge", None, _("_Merge"),
                                         None, _("Merge"),
                                         self.on_bazaar_merge)])
        
        self._action_group.add_actions([("BazaarUpdate", None, _("Up_date"),
                                         None, _("Update"),
                                         self.on_bazaar_update)])
        self._action_group.add_actions([("BazaarStatus", None, _("_Status"),
                                         None, _("Status"),
                                         self.on_bazaar_status)])
        
        self._action_group.add_actions([("BazaarMVFile", None, _("_Move file"),
                                         None, _("Move file"),
                                         self.on_bazaar_move)])
        
        self._action_group.add_actions([("BazaarDiff", None, _("_View diff"),
                                         None, _("View diff"),
                                         None)])
        
        self._action_group.add_actions([("BazaarDiffCurrent", None, _("Cu_rrent file"),
                                         None, _("Current file"),
                                         None)])
        action = self._action_group.get_action("BazaarDiffCurrent")
        action = action.connect("activate",self.on_bazaar_diff,'current')
        
        self._action_group.add_actions([("BazaarDiffBranch", None, _("Bra_nch"),
                                         None, _("Branch"),
                                         None)])
        action = self._action_group.get_action("BazaarDiffBranch")
        action = action.connect("activate",self.on_bazaar_diff,'branch')
        
        
        self._action_group.add_actions([("BazaarLog", None, _("Show _log"),
                                         None, _("Show log"),
                                         None)])
        
        self._action_group.add_actions([("BazaarLogFile", None, _("_File log"),
                                         None, _("Status"),
                                         None)])
        action = self._action_group.get_action("BazaarLogFile")
        action = action.connect("activate",self.on_bazaar_log,'file')
        
        
        self._action_group.add_actions([("BazaarLogDir", None, _("_Directory log"),
                                         None, _("Directory log"),
                                         None)])
        action = self._action_group.get_action("BazaarLogDir")
        action = action.connect("activate",self.on_bazaar_log,'dir')
        
        self._action_group.add_actions([("BazaarLogBranch", None, _("_Branch log"),
                                         None, _("Branch log"),
                                         None)])
        

                                        
        action = self._action_group.get_action("BazaarLogBranch")
        action = action.connect("activate",self.on_bazaar_log,'branch')
        
        self._action_group.add_actions([("BazaarRevert", None, _("_Revert"),
                                         None, _("Revert"),
                                         self.on_bazaar_revert)])
        self._action_group.add_actions([("BazaarIgnoreCurrent", None, _("I_gnore current file"),
                                         None, _("Ignore current file"),
                                         self.on_bazaar_ignorecurrent)])
        
        self._action_group.add_actions([("BazaarViz", None, _("View _history"),
                                         None, _("View history"),
                                         self.on_bazaar_viz)])
        self._action_group.add_actions([("BazaarCheck", None, _("Chec_k"),
                                         None, _("Check"),
                                         self.on_bazaar_check)])

        self._action_group.add_actions([("BazaarConflicts", None, _("Vie_w conflicts"),
                                         None, _("View conflicts"),
                                         self.on_bazaar_conflicts)])
        self._action_group.add_actions([("BazaarPatch", None, _("Mana_ge patch"),
                                         None, _("Manage patch"),
                                         self.on_bazaar_patch)])
        self._action_group.add_actions([("BazaarTags", None, _("Manage _tags"),
                                         None, _("Manage tags"),
                                         self.on_bazaar_tags)])
        
        self._action_group.add_actions([("BazaarConfig", None, _("Bazaar Configuration"),
                                         None, _("Bazaar Configuration"),
                                         self.on_bazaar_config)])
        
        menutool = MenuToolAction("BazaarToolButton", "Bazaar",
                                          "Bazaar", None)
        self._action_group.add_action(menutool)


        # Insert the action group
        manager.insert_action_group(self._action_group, -1)

        # Merge the UI
        self._ui_id = manager.add_ui_from_string(ui_str)

        # create toolbuton with menu
        toolbuton = manager.get_widget("/ToolBar/BazaarToolButton")


        menu = manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu")
        menuimg = gtk.Image()
        menuimg.set_from_file(self._mypath+"/img/bzr-icon-16.png")

        menu.set_image(menuimg)
        toolbuton.set_menu(menu.get_submenu())
        
        
        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/push16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarPush").set_image(img)

        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/commit16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarCommit").set_image(img)

        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/diff16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarDiff").set_image(img)
        
        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/log16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarLog").set_image(img)

        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/pull16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarPull").set_image(img)
        
        img = gtk.Image()
        img.set_from_file(self._mypath+"/img/icons/refresh16.png")
        manager.get_widget("/MenuBar/ToolsMenu/BazaarMenu/BazaarMenu/BazaarUpdate").set_image(img)
        
        
        #toolbuton.add_accelerator('activate',manager.get_accel_group(), ord('q'),"<Control>a",None)

        
    def _remove_menu(self):
        # Get the GtkUIManager
        manager = self._window.get_ui_manager()

        # Remove the ui
        manager.remove_ui(self._ui_id)

        # Remove the action group
        manager.remove_action_group(self._action_group)

        # Make sure the manager updates
        manager.ensure_update()

    def _remove_statusbar(self):
        gtk.Bin.remove(self._window.get_statusbar(),self._statusframe)
        # I spent 30min to get this f*****g thing done :)

    def _statusbar_draw(self,msg = ""):
        if self._statusframe == None:
            status = self._window.get_statusbar()
            self._statusframe = gtk.Frame()
            status.add(self._statusframe)
            hb = gtk.HBox()
            self.status_img = gtk.Image()

            self.status_img.set_from_file(self._mypath+"/img/bzr-icon-16_no.png")

            self._statuslbl = gtk.Label("")
            self._statuslbl.set_alignment(0,0)
            hb.pack_start(self.status_img)
            hb.pack_start(self._statuslbl)
            self._statusframe.add(hb)
            self._statusframe.show_all()

        if self._get_is_active_branch(self._get_active_doc_dir()) is not None:
            self.status_img.set_from_file(self._mypath+"/img/bzr-icon-16.png")
            if msg == "":
                msg = "This file is in an active bzr directory"
        else:
            if msg == "":
                msg = "This file isn't in an active bzr directory"
            self.status_img.set_from_file(self._mypath+"/img/bzr-icon-16_no.png")


        self._statuslbl.set_text(msg)

    def _set_menu_sensitive(self,isBranch):

        manager = self._window.get_ui_manager()
        toolbuton = manager.get_widget("/ToolBar/BazaarToolButton")
        img = gtk.Image()
        if isBranch:
            for menupath in self._menu_paths_bzr:
                manager.get_widget(menupath).set_sensitive(True)
            
            for menupath in self._menu_paths_nobzr:
                manager.get_widget(menupath).set_sensitive(False)            

            img.set_from_file(self._mypath+"/img/bzr-icon-24.png")

        else:
            for menupath in self._menu_paths_bzr:
                manager.get_widget(menupath).set_sensitive(False)
            
            for menupath in self._menu_paths_nobzr:
                manager.get_widget(menupath).set_sensitive(True)            

            img.set_from_file(self._mypath+"/img/bzr-icon-24_no.png")

        toolbuton.set_icon_widget(img)
        img.show()


    # Private data functions
    def _get_active_doc_dir(self):

        doc = self._window.get_active_document()
        if doc == None:
            return None
        else:
            docuri = doc.get_uri_for_display() # FIXME: is this the right one? or should i clean up get_uri?
            if os.path.exists(docuri):
                localpath = os.path.dirname(docuri)
                return localpath
            else:
                return None

    def _get_is_active_branch(self,dir):
        if not dir:
            return False
        try:
            wt = workingtree.WorkingTree.open_containing(dir)
        except:
            return False
        else:
            return True

    def _check_bzrgtk(self,command):
        if not bzrgtk:
            dialogs._bzrErrorDialog("Can't run %s command. bzr-gtk plugin needs to be installed in " \
            "your system" % command,"Goto http://launchpad.net/bzr-gtk and install the plugin")
            return False
        else:
            return True



class BazaarPlugin(gedit.Plugin):
    def __init__(self):
        gedit.Plugin.__init__(self)
        self._instances = {}

    def activate(self, window):
        self._instances[window] = BazaarWindowHelper(self, window)

    def deactivate(self, window):
        self._instances[window].deactivate()
        del self._instances[window]

    def update_ui(self, window):
        self._instances[window].update_ui()
            